/*
 * This file is part of JGrasstools (http://www.jgrasstools.org)
 * (C) HydroloGIS - www.hydrologis.com 
 * 
 * JGrasstools is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package eu.hydrologis.utils;

import java.io.File;
import java.io.FileInputStream;
import java.io.FilenameFilter;
import java.io.IOException;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Properties;

import models.GeopapProject;
import models.GpsImage;
import models.GpsLog;
import models.GpsNote;

import org.joda.time.DateTime;
import org.joda.time.format.DateTimeFormat;
import org.joda.time.format.DateTimeFormatter;

public class Utilities {

    public static DateTimeFormatter imagePropertyFormatter = DateTimeFormat.forPattern("yyyyMMdd_HHmmss"); //$NON-NLS-1$
    public static DateTimeFormatter dateTimeFormatter = DateTimeFormat.forPattern("yyyy-MM-dd HH:mm:ss"); //$NON-NLS-1$
    public static DecimalFormat altimFormatter = new DecimalFormat("###0");

    /**
     * Get the folder containing all projects as a java file (not to be used in
     * html).
     * 
     * @return the project folder file.
     */
    public static File getAllProjectsFolder() {
        File file = new File("./public/geopapprojects");
        if (!file.exists()) {
            file.mkdir();
        }
        return file;
    }

    public static File getChartsFolder() {
        File file = new File("./public/charts");
        if (!file.exists()) {
            file.mkdir();
        }
        return file;
    }

    /**
     * Get a project folder by the name as a java file (not to be used in html).
     * 
     * @param userName
     *            the name of the user.
     * @param name
     *            the project name.
     * @return the project folder.
     */
    public static File getProjectFolderByName( String userName, String name ) {
        File projectsFolder = getAllProjectsFolder();
        File userFolder = getUserFolder(userName, projectsFolder);
        File project = new File(userFolder, name);
        if (project.exists()) {
            return project;
        }
        return null;
    }

    public static File getUserFolder( String userName, File projectsFolder ) {
        File userFolder = new File(projectsFolder, userName);
        if (!userFolder.exists()) {
            userFolder.mkdir();
        }
        return userFolder;
    }

    public static File getDbfile( File projectFolder ) {
        File dbFile = new File(projectFolder, "geopaparazzi.db");

        return dbFile;
    }

    public static File getDescriptionFile( File projectFolder ) {
        File descriptionFile = new File(projectFolder, "description");
        return descriptionFile;
    }

    public static File getMediaFolder( File projectFolder ) {
        File mediaFolder = new File(projectFolder, "media");
        return mediaFolder;
    }

    /**
     * Get the list of {@link GpsImage}s containing the path for use in HTML.
     * 
     * @param projectFolder
     *            the project folder.
     * @return the list of images in the project.
     */
    public static List<GpsImage> getImages( File projectFolder ) {
        String user = projectFolder.getParentFile().getName();
        String projectName = projectFolder.getName();

        File mediaFile = getMediaFolder(projectFolder);
        File[] imageList = mediaFile.listFiles(new FilenameFilter(){
            public boolean accept( File dir, String name ) {
                return name.toLowerCase().endsWith(".jpg");
            }
        });

        List<GpsImage> images = new ArrayList<GpsImage>();
        if (imageList != null && imageList.length > 0) {
            for( File imageFile : imageList ) {
                try {
                    GpsImage image = new GpsImage();

                    image.path = "geopapprojects/" + user + "/" + projectName + "/media/" + imageFile.getName();

                    // info
                    Properties locationProperties = new Properties();
                    File propertiesFile = FileUtilities.substituteExtention(imageFile, "properties");
                    locationProperties.load(new FileInputStream(propertiesFile));
                    String azimuthString = locationProperties.getProperty("azimuth"); //$NON-NLS-1$
                    String latString = locationProperties.getProperty("latitude"); //$NON-NLS-1$
                    String lonString = locationProperties.getProperty("longitude"); //$NON-NLS-1$
                    String altimString = locationProperties.getProperty("altim"); //$NON-NLS-1$
                    String dateTimeString = locationProperties.getProperty("utctimestamp"); //$NON-NLS-1$

                    if (azimuthString != null)
                        image.azimuth = azimuthString;
                    if (altimString != null) {
                        double altim = Double.parseDouble(altimString);
                        image.altim = altimFormatter.format(altim);
                    }
                    image.lat = Double.parseDouble(latString);
                    image.lon = Double.parseDouble(lonString);

                    DateTime dateTime = imagePropertyFormatter.parseDateTime(dateTimeString);
                    String newDateTimeString = dateTime.toString(dateTimeFormatter);
                    image.dateTime = newDateTimeString;

                    images.add(image);
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }

        return images;
    }

    static {
        try {
            System.out.println("LOADING SQLITE " + new Date());
            Class.forName("org.sqlite.JDBC");
        } catch (ClassNotFoundException e) {
            e.printStackTrace();
        }
    }

    /**
     * Get {@link GpsNote}s from a project's database.
     * 
     * @param projectFolder
     *            the project folder.
     * @return the {@link List} of {@link GpsNote}s.
     * @throws Exception
     */
    public static List<GpsNote> getNotes( File projectFolder ) throws Exception {
        List<GpsNote> notes = new ArrayList<GpsNote>();

        File dbFile = getDbfile(projectFolder);

        Connection connection = null;
        Statement statement = null;
        try {

            // create a database connection
            connection = DriverManager.getConnection("jdbc:sqlite:" + dbFile.getAbsolutePath());
            statement = connection.createStatement();
            statement.setQueryTimeout(30); // set timeout to 30 sec.

            ResultSet rs = statement.executeQuery("select lat, lon, altim, ts, text from notes");
            while( rs.next() ) {
                double lat = rs.getDouble("lat");
                double lon = rs.getDouble("lon");
                double altim = rs.getDouble("altim");
                String dateTimeString = rs.getString("ts");
                String text = rs.getString("text");

                GpsNote note = new GpsNote();
                note.lat = String.valueOf(lat);
                note.lon = String.valueOf(lon);
                note.altim = altimFormatter.format(altim);
                note.dateTime = dateTimeString;
                note.text = text;
                notes.add(note);
            }
            System.out.println();
        } finally {
            try {
                if (statement != null)
                    statement.close();
                if (connection != null)
                    connection.close();
            } catch (SQLException e) {
                // connection close failed.
                e.printStackTrace();
            }
        }

        return notes;
    }

    /**
     * Get the logs from the project.
     * 
     * @param projectFolder
     *            the folder.
     * @return the list of {@link GpsLog}s.
     * @throws Exception
     */
    public static List<GpsLog> getLogs( File projectFolder ) throws Exception {
        List<GpsLog> logsList = new ArrayList<GpsLog>();
        File dbFile = getDbfile(projectFolder);
        Connection connection = null;
        try {
            connection = DriverManager.getConnection("jdbc:sqlite:" + dbFile.getAbsolutePath());
            Statement statement = null;
            try {
                // create a database connection
                statement = connection.createStatement();
                statement.setQueryTimeout(30); // set timeout to 30 sec.

                // first get the logs
                ResultSet rs = statement.executeQuery("select _id, startts, endts, text from gpslogs");
                while( rs.next() ) {
                    long id = rs.getLong("_id");

                    String startDateTimeString = rs.getString("startts");
                    String endDateTimeString = rs.getString("endts");
                    String text = rs.getString("text");

                    GpsLog log = new GpsLog();
                    log.id = id;
                    log.startTime = startDateTimeString;
                    log.endTime = endDateTimeString;
                    log.text = text;
                    logsList.add(log);
                }
            } finally {
                try {
                    if (statement != null)
                        statement.close();
                } catch (SQLException e) {
                    // connection close failed.
                    e.printStackTrace();
                }
            }

            for( GpsLog log : logsList ) {
                Statement newStatement = null;
                try {
                    long logId = log.id;
                    String query = "select lat, lon, altim, ts from gpslog_data where logid = " + logId + " order by ts";

                    newStatement = connection.createStatement();
                    newStatement.setQueryTimeout(30);
                    ResultSet result = newStatement.executeQuery(query);

                    while( result.next() ) {
                        double lat = result.getDouble("lat");
                        double lon = result.getDouble("lon");
                        double altim = result.getDouble("altim");
                        String dateTimeString = result.getString("ts");

                        GpsNote gPoint = new GpsNote();
                        gPoint.lon = String.valueOf(lon);
                        gPoint.lat = String.valueOf(lat);
                        gPoint.altim = String.valueOf(altim);
                        gPoint.dateTime = dateTimeString;
                        log.points.add(gPoint);
                    }
                } finally {
                    try {
                        if (newStatement != null)
                            newStatement.close();
                    } catch (SQLException e) {
                        // connection close failed.
                        e.printStackTrace();
                    }
                }
            }
        } finally {
            if (connection != null)
                connection.close();
        }
        return logsList;
    }

    public static GpsLog getLogById( File projectFolder, long logId ) throws Exception {
        File dbFile = getDbfile(projectFolder);
        GpsLog log = null;
        Connection connection = null;
        try {
            connection = DriverManager.getConnection("jdbc:sqlite:" + dbFile.getAbsolutePath());
            Statement statement = null;
            try {
                // create a database connection
                statement = connection.createStatement();
                statement.setQueryTimeout(30); // set timeout to 30 sec.

                // first get the logs
                ResultSet rs = statement.executeQuery("select _id, startts, endts, text from gpslogs where _id = " + logId);
                while( rs.next() ) {
                    long id = rs.getLong("_id");

                    String startDateTimeString = rs.getString("startts");
                    String endDateTimeString = rs.getString("endts");
                    String text = rs.getString("text");

                    log = new GpsLog();
                    log.id = id;
                    log.startTime = startDateTimeString;
                    log.endTime = endDateTimeString;
                    log.text = text;
                }
            } finally {
                try {
                    if (statement != null)
                        statement.close();
                } catch (SQLException e) {
                    // connection close failed.
                    e.printStackTrace();
                }
            }

            if (log == null) {
                return null;
            }

            Statement newStatement = null;
            try {
                String query = "select lat, lon, altim, ts from gpslog_data where logid = " + logId + " order by ts";

                newStatement = connection.createStatement();
                newStatement.setQueryTimeout(30);
                ResultSet result = newStatement.executeQuery(query);

                while( result.next() ) {
                    double lat = result.getDouble("lat");
                    double lon = result.getDouble("lon");
                    double altim = result.getDouble("altim");
                    String dateTimeString = result.getString("ts");

                    GpsNote gPoint = new GpsNote();
                    gPoint.lon = String.valueOf(lon);
                    gPoint.lat = String.valueOf(lat);
                    gPoint.altim = String.valueOf(altim);
                    gPoint.dateTime = dateTimeString;
                    log.points.add(gPoint);
                }
            } finally {
                try {
                    if (newStatement != null)
                        newStatement.close();
                } catch (SQLException e) {
                    // connection close failed.
                    e.printStackTrace();
                }
            }
        } finally {
            if (connection != null)
                connection.close();
        }
        return log;
    }

    /**
     * Get the logs description from the project.
     * 
     * @param projectFolder
     *            the folder.
     * @return the list of {@link GpsLog}s, but without data in them.
     * @throws Exception
     */
    public static List<GpsLog> getLogsDescriptions( File projectFolder ) throws Exception {
        List<GpsLog> logsList = new ArrayList<GpsLog>();
        File dbFile = getDbfile(projectFolder);
        Connection connection = null;
        try {
            connection = DriverManager.getConnection("jdbc:sqlite:" + dbFile.getAbsolutePath());
            Statement statement = null;
            try {
                // create a database connection
                statement = connection.createStatement();
                statement.setQueryTimeout(30); // set timeout to 30 sec.

                // first get the logs
                ResultSet rs = statement.executeQuery("select _id, startts, endts, text from gpslogs");
                while( rs.next() ) {
                    long id = rs.getLong("_id");

                    String startDateTimeString = rs.getString("startts");
                    String endDateTimeString = rs.getString("endts");
                    String text = rs.getString("text");

                    GpsLog log = new GpsLog();
                    log.id = id;
                    log.startTime = startDateTimeString;
                    log.endTime = endDateTimeString;
                    log.text = text;
                    logsList.add(log);
                }
            } finally {
                try {
                    if (statement != null)
                        statement.close();
                } catch (SQLException e) {
                    // connection close failed.
                    e.printStackTrace();
                }
            }

        } finally {
            if (connection != null)
                connection.close();
        }
        return logsList;
    }

    public static double pythagoras( double d1, double d2 ) {
        return Math.sqrt(Math.pow(d1, 2.0) + Math.pow(d2, 2.0));
    }

    public static boolean matches( GeopapProject project, String filter ) {
        if (project.name.toLowerCase().contains(filter.toLowerCase()))
            return true;
        if (project.description.toLowerCase().contains(filter.toLowerCase()))
            return true;
        if (project.userName.toLowerCase().contains(filter.toLowerCase()))
            return true;
        if (project.date.toLowerCase().contains(filter.toLowerCase()))
            return true;
        return false;
    }
}
